# Generated by Django 4.2.21 on 2025-05-27 00:35

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("auth", "0012_alter_user_first_name_max_length"),
        ("authentication", "0024_alter_userprofile_language"),
    ]

    operations = [
        migrations.CreateModel(
            name="General",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
            ],
            options={
                "permissions": (
                    ("basic_access", "Can access this app"),
                    ("editor_access", "Can edit this app"),
                ),
                "managed": False,
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="Log",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("user", models.TextField(blank=True, default="", null=True)),
                ("action", models.TextField(blank=True, default="", null=True)),
                ("details", models.TextField(blank=True, default="", null=True)),
                ("created", models.DateTimeField(auto_now_add=True)),
            ],
        ),
        migrations.CreateModel(
            name="Page",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "title",
                    models.CharField(
                        help_text="Required: Displayed above content and in menu",
                        max_length=255,
                    ),
                ),
                (
                    "slug",
                    models.CharField(
                        help_text="Required: Indentifier for path and links",
                        max_length=255,
                    ),
                ),
                (
                    "priority",
                    models.IntegerField(
                        default=10,
                        help_text="Required: Sort order for pages on the same path",
                    ),
                ),
                ("content", models.TextField(blank=True, default="", null=True)),
                (
                    "groups",
                    models.ManyToManyField(
                        blank=True,
                        default=None,
                        help_text="Optional: Restrict Page to specific Groups",
                        related_name="+",
                        to="auth.group",
                        verbose_name="groups",
                    ),
                ),
                (
                    "parent",
                    models.ForeignKey(
                        blank=True,
                        help_text="Required: Oragnise page hierarchy",
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="children",
                        to="evewiki.page",
                    ),
                ),
                (
                    "states",
                    models.ManyToManyField(
                        blank=True,
                        default=None,
                        help_text="Optional: Restrict Page to specific States",
                        related_name="+",
                        to="authentication.state",
                        verbose_name="states",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Setting",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "hierarchy_max_display_depth",
                    models.IntegerField(
                        default=10,
                        help_text="Limit the depth of the tree for the hierarchy on the main display",
                    ),
                ),
                (
                    "max_versions",
                    models.IntegerField(
                        default=1000,
                        help_text="No one has infinite disk space, a sensible limit which can be modified to clear down the history",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="PageVersion",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("content", models.TextField(blank=True, default="", null=True)),
                ("created", models.DateTimeField(auto_now_add=True)),
                (
                    "page",
                    models.ForeignKey(
                        blank=True,
                        help_text="Page being edited",
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="evewiki.page",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        blank=True,
                        help_text="User editing the page",
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="authentication.userprofile",
                    ),
                ),
            ],
        ),
    ]



from django.db import models


class Setting(models.Model):
    """
    Settings and feature flags.
    """

    hierarchy_max_display_depth = models.IntegerField(
        default=10,
        null=False,
        help_text="Limit the depth of the tree for the hierarchy on the main display",
    )

    max_versions = models.IntegerField(
        default=1000,
        null=False,
        help_text="No one has infinite disk space, a sensible limit which can be modified to clear down the history",
    )

    def get_settings():
        """
        Returns the Setting instance with the lowest id,
        or a default Setting instance (not saved) if none exist.
        """
        setting = Setting.objects.order_by("id").first()
        if setting is not None:
            return setting

        return Setting(
            hierarchy_max_display_depth=Setting._meta.get_field(
                "hierarchy_max_display_depth"
            ).get_default(),
            max_versions=Setting._meta.get_field("max_versions").get_default(),
        )



from django.db import models

from allianceauth.authentication.models import UserProfile

from .pages import Page
from .settings import Setting


class PageVersion(models.Model):
    """
    Previous edits have value
    """

    page = models.ForeignKey(
        Page,
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
        help_text="Page being edited",
    )

    user = models.ForeignKey(
        UserProfile,
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
        help_text="User editing the page",
    )

    content = models.TextField(default="", blank=True, null=True)

    created = models.DateTimeField(auto_now_add=True)

    def save(self, *args, **kwargs):
        """
        Clean up old versions when saving
        """
        super().save(*args, **kwargs)
        if self.page_id:
            # Get the IDs of old versions to delete
            settings = Setting.get_settings()
            old_version_ids = list(
                PageVersion.objects.filter(page=self.page)
                .order_by("-created")
                .values_list("id", flat=True)[settings.max_versions :]
            )
            if old_version_ids:
                PageVersion.objects.filter(id__in=old_version_ids).delete()



"""Tasks."""

from celery import shared_task

from allianceauth.services.hooks import get_extension_logger

logger = get_extension_logger(__name__)


@shared_task
def my_task():
    """An example task."""



import re
from html import escape

from django.contrib.auth.models import Group
from django.core.exceptions import ValidationError
from django.db import models

from allianceauth.authentication.models import State


class General(models.Model):
    """A meta model for app permissions."""

    class Meta:
        managed = False
        default_permissions = ()
        permissions = (
            ("basic_access", "Can access this app"),
            ("editor_access", "Can edit this app"),
        )


class Page(models.Model):
    """
    Pages in a hierarchy, identifiable by slugs with a light touch of folder sorting.
    """

    title = models.CharField(
        max_length=255,
        null=False,
        help_text="Required: Displayed above content and in menu",
    )

    parent = models.ForeignKey(
        "self",
        on_delete=models.PROTECT,
        related_name="children",
        null=True,
        blank=True,
        help_text="Required: Oragnise page hierarchy",
    )

    slug = models.CharField(
        max_length=255, help_text="Required: Indentifier for path and links"
    )

    priority = models.IntegerField(
        default=10, help_text="Required: Sort order for pages on the same path"
    )

    states = models.ManyToManyField(
        State,
        default=None,
        blank=True,
        related_name="+",
        verbose_name=("states"),
        help_text="Optional: Restrict Page to specific States",
    )

    groups = models.ManyToManyField(
        Group,
        default=None,
        blank=True,
        related_name="+",
        verbose_name=("groups"),
        help_text="Optional: Restrict Page to specific Groups",
    )

    content = models.TextField(default="", blank=True, null=True)

    def __str__(self):
        return self.title

    def clean(self):

        # Force slugs to be alphanumber lowercase with dashes
        self.slug = self.slug.lower().replace(" ", "-")
        self.slug = re.sub(r"[^a-z0-9\-]", "", self.slug)

        # Check slug is unique for this path
        # Prepare the queryset for siblings (excluding self if updating)
        siblings_qs = Page.objects.filter(parent=self.parent)
        if self.pk is not None:
            siblings_qs = siblings_qs.exclude(pk=self.pk)

        sibling_slugs = list(siblings_qs.values_list("slug", flat=True))
        if self.slug in sibling_slugs:
            raise ValidationError(
                f'A slug with "{self.slug}" already exists ar this path'
            )

        # Check circular reference to self
        if self.parent and self.pk and self.parent.pk == self.pk:
            raise ValidationError("A page cannot be its own parent.")

    @property
    def path(self):
        segments = []
        node = self
        while node is not None:
            segments.append(node.slug)
            node = node.parent
        return "/".join(reversed(segments))

    @property
    def summary(self):
        """
        Generates a summary of headers from Markdown text as a nested HTML list of links.
        """
        header_pattern = re.compile(r"^(#{1,6})\s+(.*)", re.MULTILINE)
        headers = []

        for match in header_pattern.finditer(self.content):
            hashes, title = match.groups()
            level = len(hashes)
            anchor = re.sub(r"[^\w\s-]", "", title).strip().lower()
            anchor = re.sub(r"\s+", "-", anchor)
            headers.append((level, title, anchor))

        # Build nested HTML list
        html = []
        prev_level = 0
        for level, title, anchor in headers:
            while prev_level < level:
                html.append("<ul>")
                prev_level += 1
            while prev_level > level:
                html.append("</ul>")
                prev_level -= 1
            html.append(f'<li><a href="#{escape(anchor)}">{escape(title)}</a></li>')
        while prev_level > 0:
            html.append("</ul>")
            prev_level -= 1

        return "\n".join(html)

    @classmethod
    def list(cls, pages=None, parent=None, level=0):
        """
        Returns a flat list of pages with titles indented by hierarchy level.
        Returns: list of tuple
        """
        if pages is None:
            pages = list(cls.objects.all().select_related("parent"))

        flat = []
        children = [p for p in pages if p.parent_id == (parent.id if parent else None)]
        children.sort(key=lambda p: p.priority)

        for child in children:
            indent = "\u2003" * level
            display = f"{indent}{child.title} [/{child.path}]"
            flat.append((child.id, display))
            flat.extend(cls.list(pages, child, level + 1))

        if parent is None and level == 0:
            flat.insert(0, ("", "/"))

        return flat

    @classmethod
    def tree(cls, user=None, pages=None, parent=None, depth=0):
        """
        Recursively builds a tree of pages as nested dicts,
        ordered by priority, with 'path' property.
        Returns: List[Dict]
        """
        if pages is None:
            pages = list(cls.objects.all().select_related("parent"))

        tree = []
        children = [p for p in pages if p.parent_id == (parent.id if parent else None)]
        children.sort(key=lambda p: p.priority)

        for child in children:
            node = {
                "id": child.id,
                "title": child.title,
                "slug": child.slug,
                "priority": child.priority,
                "path": getattr(child, "path", None),
                "depth": depth,
                "children": cls.tree(user, pages, child, depth=depth + 1),
            }
            if child.user_access(user=user):
                tree.append(node)
        return tree

    @classmethod
    def get_by_path(cls, path: str, user):
        """
        Find an id for a given path
        Returns: Page id or None
        """
        path = path.split("/")
        parent = None
        page = None
        for slug in path:
            try:
                page = cls.objects.filter(slug=slug, parent=parent).first()
            except cls.DoesNotExist:
                return None
            parent = page

        if page and not page.user_access(user=user):
            return None

        return page if page else None

    def user_access(self, user):

        # Admin has full access
        if user.profile.state.name == "Admin":
            return True

        # Editor has full access
        if user.has_perm("evewiki.editor_access"):
            return True

        access = False

        # If no groups or states assigned then yes
        groups_empty = not self.groups.exists()
        states_empty = not self.states.exists()
        if groups_empty and states_empty:
            access = True

        # If any of the user's groups are allowed access
        user_has_group = False
        if any(
            group in self.groups.all()
            for group in user.groups.values_list("name", flat=True)
        ):
            access = True
            user_has_group = True

        # If the user's states is in the states list
        user_has_state = False
        if self.states.filter(name=user.profile.state.name).exists():
            access = True
            user_has_state = True

        # If Page has both User needs both
        if not groups_empty and not states_empty:
            access = False
            if user_has_group and user_has_state:
                access = True

        return access



"""Views."""

from django.contrib import messages
from django.contrib.auth.decorators import login_required, permission_required
from django.core.handlers.wsgi import WSGIRequest
from django.http import HttpResponse
from django.shortcuts import redirect, render

from allianceauth.authentication.models import UserProfile

from .forms import PageForm
from .models.logs import Log
from .models.page_versions import PageVersion
from .models.pages import Page
from .models.settings import Setting


@login_required
@permission_required("evewiki.basic_access")
def index(request: WSGIRequest, unknown_path: str = "/"):
    """Render index view."""

    try:

        settings = Setting.get_settings()

        is_editor = False
        if request.user.has_perm("evewiki.editor_access"):
            is_editor = True

        # Control light/dark mode on md editor
        dark_mode = False
        themes_to_use_dark_mode = [
            "allianceauth.theme.darkly.auth_hooks.DarklyThemeHook"
        ]
        user = UserProfile.objects.filter(user=request.user).first()
        if user.theme in themes_to_use_dark_mode:
            dark_mode = True

        # Tree is too complex to call directly froma template
        tree = Page.tree(user=request.user)

        # "Unknown" i.e. has not bee picked up by urls.py
        page = Page.get_by_path(path=unknown_path, user=request.user)

        # Load the first page by default if nothing else can be found
        # If there is no first-page, template will load a help page.
        if page is None:
            page = Page.objects.order_by("id").first()

        # Content save
        new_content = request.POST.get("content")
        if new_content is not None:
            try:
                # Save the content
                page.content = request.POST.get("content")
                page.save()
                # Record a version of the content
                page_version = PageVersion(
                    page=page,
                    user=user,
                    content=page.content,
                )
                page_version.save()
                # Log a thing happened
                Log(user=user, action=f"Page content modified: {page.title}").save()
                messages.success(request, f"Page '{page.title}' saved")
                return redirect(f"/evewiki/{page.path}")
            except Page.DoesNotExist:
                pass

        context = {
            "context-title": "context-not-title",
            "tree": tree,
            "page": page,
            "dark_mode": dark_mode,
            "settings": settings,
            "is_editor": is_editor,
        }
        return render(request, "evewiki/index.html", context)
    except Exception as e:
        print(f"setup required: {e}")
        return render(request, "evewiki/error.html")


@login_required
@permission_required("evewiki.basic_access")
def page(request: WSGIRequest) -> HttpResponse:

    try:
        if (
            not request.user.has_perm("evewiki.editor_access")
            and not request.user.profile.state.name == "Admin"
        ):
            return redirect("/evewiki")

        user = UserProfile.objects.filter(user=request.user).first()
        page = None
        page_id = request.GET.get("id") or request.POST.get("id")
        if page_id is not None:
            try:
                page = Page.objects.get(id=page_id)
            except Page.DoesNotExist:
                page = None

        if request.method == "POST":
            form = PageForm(request.POST, instance=page)
            if form.is_valid():
                saved_page = form.save()
                Log(user=user, action=f"Page Created: {saved_page.title}").save()
                messages.success(request, f"Page '{saved_page.title}' saved")
                return redirect(f"/evewiki/{saved_page.path}")
        else:
            form = PageForm(instance=page)

        page_versions = PageVersion.objects.filter(page=page).order_by("-created").all()

        context = {
            "Page": "Page",
            "page": page,
            "form": form,
            "versions": page_versions,
        }
        return render(request, "evewiki/page.html", context)
    except Exception as e:
        print(f"setup required: {e}")
        return render(request, "evewiki/error.html")


@login_required
@permission_required("evewiki.basic_access")
def page_delete(request: WSGIRequest) -> HttpResponse:

    try:
        if (
            not request.user.has_perm("evewiki.editor_access")
            and not request.user.profile.state.name == "Admin"
        ):
            return redirect("/evewiki")

        user = UserProfile.objects.filter(user=request.user).first()
        page = None
        page_id = request.GET.get("id")
        confirm_delete = request.GET.get("confirm_delete")
        if page_id is not None:
            try:
                page = Page.objects.get(id=page_id)
            except Page.DoesNotExist:
                page = None

        # Check if page has children
        children = Page.objects.filter(parent_id=page_id).all()
        if len(children) > 0:
            messages.error(request, "You cannot delete a page that has children.")

        # Confirm the confirmation that this user definitely wants to delete
        if confirm_delete == "true":
            messages.success(request, f"Page '{page.title}' deleted")
            page.delete()
            # Pages can be deleted, links can be broken, versions orphaned.
            # But the log is immutable
            Log(user=user, action=f"Page Deleted: {page.title}").save()
            return redirect("/evewiki/index")

        context = {"Delete Page": "Delete Page", "page": page, "children": children}
        return render(request, "evewiki/page_delete.html", context)
    except Exception as e:
        print(f"setup required: {e}")
        return render(request, "evewiki/error.html")



from django.db import models


class Log(models.Model):
    """
    No relationships just a log.
    """

    user = models.TextField(default="", blank=True, null=True)

    action = models.TextField(default="", blank=True, null=True)

    details = models.TextField(default="", blank=True, null=True)

    created = models.DateTimeField(auto_now_add=True)



"""Routes."""

from django.urls import path, re_path

from evewiki import views

app_name = "evewiki"

urlpatterns = [
    path("", views.index, name="index"),
    path("page", views.page, name="page"),
    path("page_delete", views.page_delete, name="page_delete"),
    # Send unknown routes to the index and seach for the slug
    re_path(r"^(?P<unknown_path>.*)$", views.index),
]



"""Admin site."""

from django.contrib import admin

from .models.logs import Log
from .models.page_versions import PageVersion
from .models.pages import Page
from .models.settings import Setting

# from django.contrib import admin

# Register your models for the admin site here.
admin.site.register(Page)
admin.site.register(Setting)
admin.site.register(PageVersion)
admin.site.register(Log)



from django import forms

from .models.pages import Page


class PageForm(forms.ModelForm):

    try:
        # Customised ddl inferring additional context via text-indentation
        parent = forms.ChoiceField(
            choices=Page.list(),
            required=False,
            label="Path",
            help_text=Page._meta.get_field("parent").help_text,
        )

        class Meta:
            model = Page
            fields = ["title", "parent", "slug", "priority", "states", "groups"]

        def clean(self):
            """
            Django needs a little help to turn the custom `parent` field back into a model.
            """
            cleaned_data = super().clean()
            parent_id = cleaned_data["parent"]
            cleaned_data["parent"] = (
                Page.objects.get(pk=int(parent_id)) if parent_id else None
            )
            return cleaned_data

    except Exception as e:
        print(f"setup required: {e}")
